@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013, 2014, 2015 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013, 2014, 2015, 2017 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -139,6 +139,29 @@ void Page::item_activated(GtkTreeView* view, GtkTreePath* path, GtkTreeViewColum
 
 //-----------------------------------------------------------------------------
 
+void Page::item_action_activated(GtkMenuItem* menuitem, DesktopAction* action)
+{
+	Launcher* launcher = get_selected_launcher();
+	if (!launcher)
+	{
+		return;
+	}
+
+	// Add to recent
+	if (remember_launcher(launcher))
+	{
+		m_window->get_recent()->add(launcher);
+	}
+
+	// Hide window
+	m_window->hide();
+
+	// Execute app
+	launcher->run(gtk_widget_get_screen(GTK_WIDGET(menuitem)), action);
+}
+
+//-----------------------------------------------------------------------------
+
 gboolean Page::view_button_press_event(GtkWidget* view, GdkEvent* event)
 {
 	GdkEventButton* event_button = reinterpret_cast<GdkEventButton*>(event);
@@ -194,10 +217,27 @@ void Page::create_context_menu(GtkTreeIter* iter, GdkEvent* event)
 	menuitem = gtk_separator_menu_item_new();
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
+	const std::vector<DesktopAction*> actions = launcher->get_actions();
+	if (!actions.empty())
+	{
+		for (std::vector<DesktopAction*>::size_type i = 0, end = actions.size(); i < end; ++i)
+		{
+			DesktopAction* action = actions[i];
+			menuitem = gtk_image_menu_item_new_with_label(action->get_name());
+			GtkWidget* image = gtk_image_new_from_icon_name(action->get_icon(), GTK_ICON_SIZE_MENU);
+			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), image);
+			g_signal_connect_slot(menuitem, "activate", &Page::item_action_activated, this, action);
+			gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+		}
+
+		menuitem = gtk_separator_menu_item_new();
+		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+	}
+
 	if (!m_window->get_favorites()->contains(launcher))
 	{
 		menuitem = gtk_image_menu_item_new_with_label(_("Add to Favorites"));
-		GtkWidget* image = gtk_image_new_from_icon_name("stock_add-bookmark", GTK_ICON_SIZE_MENU);
+		GtkWidget* image = gtk_image_new_from_icon_name("bookmark-new", GTK_ICON_SIZE_MENU);
 		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), image);
 		g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &Page::add_selected_to_favorites, this);
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
@@ -205,7 +245,7 @@ void Page::create_context_menu(GtkTreeIter* iter, GdkEvent* event)
 	else
 	{
 		menuitem = gtk_image_menu_item_new_with_label(_("Remove From Favorites"));
-		GtkWidget* image = gtk_image_new_from_stock(GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU);
+		GtkWidget* image = gtk_image_new_from_icon_name("list-remove", GTK_ICON_SIZE_MENU);
 		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), image);
 		g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &Page::remove_selected_from_favorites, this);
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
@@ -219,6 +259,13 @@ void Page::create_context_menu(GtkTreeIter* iter, GdkEvent* event)
 	g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &Page::add_selected_to_panel, this);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
+	menuitem = gtk_separator_menu_item_new();
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+
+	menuitem = gtk_menu_item_new_with_label(_("Edit Application..."));
+	g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &Page::edit_selected, this);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+
 	extend_context_menu(menu);
 
 	gtk_widget_show_all(menu);
@@ -378,6 +425,25 @@ void Page::add_selected_to_favorites()
 
 //-----------------------------------------------------------------------------
 
+void Page::edit_selected()
+{
+	Launcher* launcher = get_selected_launcher();
+	g_assert(launcher != NULL);
+
+	m_window->hide();
+
+	GError* error = NULL;
+	gchar* command = g_strconcat("exo-desktop-item-edit ", launcher->get_uri(), NULL);
+	if (g_spawn_command_line_async(command, &error) == false)
+	{
+		xfce_dialog_show_error(NULL, error, _("Unable to edit launcher."));
+		g_error_free(error);
+	}
+	g_free(command);
+}
+
+//-----------------------------------------------------------------------------
+
 void Page::remove_selected_from_favorites()
 {
 	Launcher* launcher = get_selected_launcher();
